/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package zswzg.kex.tls;

import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse für die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.anzeigeTypUmsetzungTls.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/03/09 10:55:21 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTls
{
  /**
   * DebugLogger für Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Datenmodell.
   */
  protected DataModel m_dataModel;
  /**
   * Pid des Konfigurationsobjektes vom Typ typ.anzeigeTypUmsetzungTls.
   */
  protected String m_pid;
  /**
   * Konfigurationsobjekt vom Typ typ.anzeigeQuerschnittTypUmsetzung.
   */
  protected SystemObject m_anzeigeTypUmsetzung;
  /**
   * Pid des GrundFont.
   */
  protected String m_pidGrundFont = null;
  /**
   * GrundTextAusrichtung.
   */
  protected String m_grundTextAusrichtung = "Linksbündig";
  /**
   * HashMap mit Instanzen der Klasse mit den FontUmsetzung-Informationen.
   */
  protected HashMap<String, FontUmsetzungTls> m_listeFontUmsetzungTls = new HashMap<String, FontUmsetzungTls>();
  /**
   * HashMap mit Instanzen der Klasse mit den TextAusrichtungUmsetzung-Informationen.
   */
  protected HashMap<String, TextAusrichtungUmsetzungTls> m_listeTextAusrichtungUmsetzungTls = new HashMap<String, TextAusrichtungUmsetzungTls>();
  /**
   * HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlüssel Pid des Zeichen.
   */
  protected HashMap<String, ZeichenUmsetzungTls> m_listeZeichenUmsetzungTlsZeichen = new HashMap<String, ZeichenUmsetzungTls>();
  /**
   * HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlüssel Code des Zeichen.
   */
  protected HashMap<Integer, ZeichenUmsetzungTls> m_listeZeichenUmsetzungTlsCode = new HashMap<Integer, ZeichenUmsetzungTls>();
  /**
   * HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls mit dem Schlüssel Pid des WvzInhalt.
   */
  protected HashMap<String, WvzInhaltUmsetzungTls> m_listeWvzInhaltUmsetzungTlsWvzInhalt = new HashMap<String, WvzInhaltUmsetzungTls>();
  /**
   * HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls mit dem Schlüssel Code des WvzInhalt.
   */
  protected HashMap<Integer, WvzInhaltUmsetzungTls> m_listeWvzInhaltUmsetzungTlsCode = new HashMap<Integer, WvzInhaltUmsetzungTls>();
  /**
   * Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   */
  protected Vector<GlobalUmsetzung> m_listeGlobalUmsetzung = new Vector<GlobalUmsetzung>();

  /**
   * Konstruktor mit Übergabe des Daten-Modell, der Pid der AnzeigeTypUmsetzung und dem Konfigurationsobjekt vom Typ typ.anzeigeTypUmsetzung.
   *
   * @param dataModel Daten-Modell.
   * @param pid Pid der AnzeigeTypUmsetzung.
   * @param anzeigeTypUmsetzung Konfigurationsobjekt vom Typ typ.anzeigeTypUmsetzung.
   */
  public AnzeigeTypUmsetzungTls(DataModel dataModel, String pid, SystemObject anzeigeTypUmsetzung)
  {
    m_dataModel = dataModel;
    m_pid = pid;
    m_anzeigeTypUmsetzung = anzeigeTypUmsetzung;

    AttributeGroup atgAnzeigeTypUmsetzung                     = m_dataModel.getAttributeGroup("atg.anzeigeTypUmsetzung");
    AttributeGroup atgAnzeigeTypUmsetzungGrundFont            = m_dataModel.getAttributeGroup("atg.anzeigeTypUmsetzungGrundFont");
    AttributeGroup atgAnzeigeTypUmsetzungGrundTextAusrichtung = m_dataModel.getAttributeGroup("atg.anzeigeTypUmsetzungGrundTextAusrichtung");
    AttributeGroup atgAnzeigeTypUmsetzungTls                  = m_dataModel.getAttributeGroup("atg.anzeigeTypUmsetzungTls");
    AttributeGroup atgZeichenUmsetzung                        = m_dataModel.getAttributeGroup("atg.zeichenUmsetzung");
    AttributeGroup atgZeichenUmsetzungTls                     = m_dataModel.getAttributeGroup("atg.zeichenUmsetzungTls");
    @SuppressWarnings("unused")
    AttributeGroup atgFontUmsetzung                           = m_dataModel.getAttributeGroup("atg.fontUmsetzung");
    @SuppressWarnings("unused")
    AttributeGroup atgFontUmsetzungTls                        = m_dataModel.getAttributeGroup("atg.fontUmsetzungTls");
    AttributeGroup atgWvzInhaltUmsetzung                      = m_dataModel.getAttributeGroup("atg.wvzInhaltUmsetzung");
    AttributeGroup atgWvzInhaltUmsetzungTls                   = m_dataModel.getAttributeGroup("atg.wvzInhaltUmsetzungTls");

    Data datenAtgAnzeigeTypUmsetzungGrundFont = anzeigeTypUmsetzung.getConfigurationData(atgAnzeigeTypUmsetzungGrundFont);

    if (datenAtgAnzeigeTypUmsetzungGrundFont != null)
    {
    	m_pidGrundFont = datenAtgAnzeigeTypUmsetzungGrundFont.getReferenceValue("FontUmsetzungReferenz").getValueText();
    }

    Data datenAtgAnzeigeTypUmsetzungGrundTextAusrichtung = anzeigeTypUmsetzung.getConfigurationData(atgAnzeigeTypUmsetzungGrundTextAusrichtung);

    if (datenAtgAnzeigeTypUmsetzungGrundTextAusrichtung != null)
    {
      m_grundTextAusrichtung = datenAtgAnzeigeTypUmsetzungGrundTextAusrichtung.getScaledValue("TextAusrichtung").getValueText();
    }

    Data datenAtgAnzeigeTypUmsetzung = anzeigeTypUmsetzung.getConfigurationData(atgAnzeigeTypUmsetzung);

    int anzahlFontUmsetzung = datenAtgAnzeigeTypUmsetzung.getReferenceArray("FontUmsetzungReferenz").getLength();

    SystemObject umsetzungFont;

    for (int i = 0; i < anzahlFontUmsetzung; i++)
    {
      umsetzungFont = datenAtgAnzeigeTypUmsetzung.getReferenceArray("FontUmsetzungReferenz").getReferenceValue(i).getSystemObject();

      FontUmsetzungTls fontUmsetzungTls = new FontUmsetzungTls(m_dataModel, umsetzungFont.getPid(), umsetzungFont);

      m_listeFontUmsetzungTls.put(umsetzungFont.getPid(), fontUmsetzungTls);
    }

    if (m_listeFontUmsetzungTls.get(m_pidGrundFont) == null)
    {
      m_debug.config("GrundFont nicht versorgt " + getPid());
    }

    int anzahlTextAusrichtungUmsetzung = datenAtgAnzeigeTypUmsetzung.getReferenceArray("TextAusrichtungUmsetzungReferenz").getLength();

    SystemObject umsetzungTextAusrichtung;

    for (int i = 0; i < anzahlTextAusrichtungUmsetzung; i++)
    {
      umsetzungTextAusrichtung = datenAtgAnzeigeTypUmsetzung.getReferenceArray("TextAusrichtungUmsetzungReferenz").getReferenceValue(i).getSystemObject();

      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls = new TextAusrichtungUmsetzungTls(m_dataModel, umsetzungTextAusrichtung.getPid(), umsetzungTextAusrichtung);

      m_listeTextAusrichtungUmsetzungTls.put(umsetzungTextAusrichtung.getPid(), textAusrichtungUmsetzungTls);
    }

    int anzahlZeichenUmsetzungReferenz = datenAtgAnzeigeTypUmsetzung.getReferenceArray("ZeichenUmsetzungReferenz").getLength();

    SystemObject umsetzungZeichenUmsetzung;
    Data datenAtgZeichenUmsetzung;
    Data datenAtgZeichenUmsetzungTls;
    int zeichencode;
    String zeichenPid;

    for (int i = 0; i < anzahlZeichenUmsetzungReferenz; i++)
    {
      umsetzungZeichenUmsetzung = datenAtgAnzeigeTypUmsetzung.getReferenceArray("ZeichenUmsetzungReferenz").getReferenceValue(i).getSystemObject();

      datenAtgZeichenUmsetzung    = umsetzungZeichenUmsetzung.getConfigurationData(atgZeichenUmsetzung);
      datenAtgZeichenUmsetzungTls = umsetzungZeichenUmsetzung.getConfigurationData(atgZeichenUmsetzungTls);

      zeichenPid  = datenAtgZeichenUmsetzung.getReferenceValue("ZeichenReferenz").getValueText();
      zeichencode = datenAtgZeichenUmsetzungTls.getUnscaledValue("Zeichencode").intValue();

//      System.out.println("ANZEIGETYPUMSETZUNGTLS Zeichenumsetzung '" + zeichenPid + "' " + zeichencode);

      ZeichenUmsetzungTls zeichenUmsetzungTls = new ZeichenUmsetzungTls(zeichenPid, zeichencode);

      m_listeZeichenUmsetzungTlsZeichen.put(zeichenPid, zeichenUmsetzungTls);
      m_listeZeichenUmsetzungTlsCode.put(zeichencode, zeichenUmsetzungTls);
    }

    int anzahlWvzInhaltUmsetzungReferenz = datenAtgAnzeigeTypUmsetzung.getReferenceArray("WvzInhaltUmsetzungReferenz").getLength();

    SystemObject umsetzungWvzInhalt;
    Data datenAtgWvzInhaltUmsetzung;
    Data datenAtgWvzInhaltUmsetzungTls;
    int code;
    String pidWvzInhalt;

    for (int i = 0; i < anzahlWvzInhaltUmsetzungReferenz; i++)
    {
      umsetzungWvzInhalt = datenAtgAnzeigeTypUmsetzung.getReferenceArray("WvzInhaltUmsetzungReferenz").getReferenceValue(i).getSystemObject();

      datenAtgWvzInhaltUmsetzung    = umsetzungWvzInhalt.getConfigurationData(atgWvzInhaltUmsetzung);
      datenAtgWvzInhaltUmsetzungTls = umsetzungWvzInhalt.getConfigurationData(atgWvzInhaltUmsetzungTls);

      pidWvzInhalt  = datenAtgWvzInhaltUmsetzung.getReferenceValue("WvzInhaltReferenz").getValueText();

      ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)datenAtgWvzInhaltUmsetzung.getReferenceValue("WvzInhaltReferenz").getSystemObject();

      AttributeGroup atgWvzInhalt = ZwischenschichtAq.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");

      Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
      String wvzInhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();

      code = datenAtgWvzInhaltUmsetzungTls.getUnscaledValue("Code").intValue();

//      System.out.println("ANZEIGETYPUMSETZUNGTLS WvzInhaltUmsetzung '" + wvzInhalt + "' " + code);

      WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = new WvzInhaltUmsetzungTls(pidWvzInhalt, wvzInhalt, code);

      m_listeWvzInhaltUmsetzungTlsWvzInhalt.put(wvzInhalt, wvzInhaltUmsetzungTls);
      m_listeWvzInhaltUmsetzungTlsCode.put(code, wvzInhaltUmsetzungTls);
    }

    Data datenAtgAnzeigeTypUmsetzungTls = anzeigeTypUmsetzung.getConfigurationData(atgAnzeigeTypUmsetzungTls);

    if (datenAtgAnzeigeTypUmsetzungTls != null)
    {
      int anzahlUmsetzungen = datenAtgAnzeigeTypUmsetzungTls.getArray("UmsetzungGesamt").getLength();

      for (int i = 0; i < anzahlUmsetzungen; i++)
      {
        GlobalUmsetzung globalUmsetzung = new GlobalUmsetzung(datenAtgAnzeigeTypUmsetzungTls.getArray("UmsetzungGesamt").getItem(i));

        m_listeGlobalUmsetzung.add(globalUmsetzung);
      }
    }
  }

  /**
   * @return the grundTextAusrichtung
   */
  public String getGrundTextAusrichtung()
  {
    return m_grundTextAusrichtung;
  }

  /**
   * @param grundTextAusrichtung the grundTextAusrichtung to set
   */
  public void setGrundTextAusrichtung(String grundTextAusrichtung)
  {
    m_grundTextAusrichtung = grundTextAusrichtung;
  }

  /**
   * Holen der Pid der AnzeigeTypUmsetzungTls.
   *
   * @return Pid der AnzeigeTypUmsetzungTls.
   */
  public String getPid()
  {
    return m_pid;
  }

  /**
   * Setzen der Pid der AnzeigeTypUmsetzungTls.
   *
   * @param pid Pid der AnzeigeTypUmsetzungTls.
   */
  public void setPid(String pid)
  {
    m_pid = pid;
  }

  /**
   * Holen des Konfigurationsobjektes vom Typ typ.anzeigeTypUmsetzung.
   *
   * @return Konfigurationsobjekt vom Typ typ.anzeigeTypUmsetzung.
   */
  public SystemObject getAnzeigeTypUmsetzung()
  {
    return m_anzeigeTypUmsetzung;
  }

  /**
   * Setzen des Konfigurationsobjektes vom Typ typ.anzeigeTypUmsetzung.
   *
   * @param anzeigeTypUmsetzung Konfigurationsobjekt vom Typ typ.anzeigeTypUmsetzung.
   */
  public void setAnzeigeTypUmsetzung(SystemObject anzeigeTypUmsetzung)
  {
    m_anzeigeTypUmsetzung = anzeigeTypUmsetzung;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlüssel Code des Zeichen.
   *
   * @return HashMap mit Instanzen der Klasse ZeichenUmsetzungTls.
   */
  public HashMap getListeZeichenUmsetzungTlsCode()
  {
    return m_listeZeichenUmsetzungTlsCode;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlüssel Pid des Zeichen.
   *
   * @return HashMap mit Instanzen der Klasse ZeichenUmsetzungTls.
   */
  public HashMap getListeZeichenUmsetzungTlsZeichen()
  {
    return m_listeZeichenUmsetzungTlsZeichen;
  }

  /**
   * Holen der Pid des Grundfont.
   *
   * @return Pid des Grundfont.
   */
  public String getPidGrundFont()
  {
    return m_pidGrundFont;
  }

  /**
   * Setzen der Pid des Grundfont.
   *
   * @param pidGrundFont Pid des Grundfont.
   */
  public void setPidGrundFont(String pidGrundFont)
  {
    m_pidGrundFont = pidGrundFont;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse FontUmsetzungTls.
   *
   * @return HashMap mit Instanzen der Klasse FontUmsetzungTls.
   */
  public HashMap<String, FontUmsetzungTls> getListeFontUmsetzungTls()
  {
    return m_listeFontUmsetzungTls;
  }

  /**
   * Setzen der HashMap mit Instanzen der Klasse FontUmsetzungTls.
   *
   * @param listeFontUmsetzungTls HashMap mit Instanzen der Klasse FontUmsetzungTls.
   */
  public void setListeFontUmsetzungTls(HashMap<String, FontUmsetzungTls> listeFontUmsetzungTls)
  {
    m_listeFontUmsetzungTls = listeFontUmsetzungTls;
  }

  /**
   * Holen des Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   *
   * @return Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   */
  public Vector getListeGlobalUmsetzung()
  {
    return m_listeGlobalUmsetzung;
  }

  /**
   * Setzen des Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   *
   * @param listeGlobalUmsetzung Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   */
  public void setListeGlobalUmsetzung(Vector<GlobalUmsetzung> listeGlobalUmsetzung)
  {
    m_listeGlobalUmsetzung = listeGlobalUmsetzung;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls mit dem Schlüssel Code des WVZ-Inhalt.
   *
   * @return HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls.
   */
  public HashMap getListeWvzInhaltUmsetzungTlsCode()
  {
    return m_listeWvzInhaltUmsetzungTlsCode;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls mit dem Schlüssel Pid des WVZ-Inhalt.
   * 
   * @return HashMap mit Instanzen der Klasse WvzInhaltUmsetzungTls.
   */
  public HashMap getListeWvzInhaltUmsetzungTlsWvzInhalt()
  {
    return m_listeWvzInhaltUmsetzungTlsWvzInhalt;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse TextAusrichtungUmsetzungTls.
   * 
   * @return HashMap mit Instanzen der Klasse TextAusrichtungUmsetzungTls.
   */
  public HashMap<String, TextAusrichtungUmsetzungTls> getListeTextAusrichtungUmsetzungTls()
  {
    return m_listeTextAusrichtungUmsetzungTls;
  }

  /**
   * Setzen der HashMap mit Instanzen der Klasse TextAusrichtungUmsetzungTls.
   * 
   * @param listeTextAusrichtungUmsetzungTls HashMap mit Instanzen der Klasse TextAusrichtungUmsetzungTls.
   */
  public void setListeTextAusrichtungUmsetzungTls(HashMap<String, TextAusrichtungUmsetzungTls> listeTextAusrichtungUmsetzungTls)
  {
    m_listeTextAusrichtungUmsetzungTls = listeTextAusrichtungUmsetzungTls;
  }  
}