/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package zswzg.kex.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;


import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse für die spezielle Umsetzung der Anzeige-Informationen für BW D-Zeile A8.
 * Diese Klasse wird über die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung überschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2012/05/23 15:47:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsBw1DZeile extends AnzeigeTypUmsetzungTlsRp1
{
  /**
   * DebugLogger für Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit Übergabe der notwendigen HashMaps.
   *
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst für alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden für alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsBw1DZeile(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }

  /**
   * Konstruktor mit Übergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsBw1DZeile(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }
}
