/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package zswzg.kex.tls;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse für die Speicherung der Informationen einer Attribut-Liste vom Typ atl.anzeigenGlobalUmsetzung.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class GlobalUmsetzung
{
  /**
   * Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   */
  protected SystemObject m_objekt = null;
  /**
   * Informationen aus dem Attribut Template der Attribut-Liste.
   */
  protected long[] m_template = null;
  /**
   * Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   */
  protected String m_umsetzungsModul = null;
  /**
   * Information aus dem Attribut AttributGruppe der Attribut-Liste.
   */
  protected String m_attributGruppe = null;
  /**
   * Bei der Umsetzung zu verwendente AttributGruppen-Aspekt-Kombinationen.
   */
  protected GlobalUmsetzungVerwendeteAttributGruppenAspekte m_verwendeteAttributGruppenAspekte = null;

  /**
   * Konstruktor mit Übergabe eines Datenobjekt mit den Attribut-Listen-Daten.
   *
   * @param data Datenobjekt mit den Attribut-Listen-Daten.
   */
  public GlobalUmsetzung(Data data)
  {
    setObjekt(data.getReferenceValue("UmsetzungsObjekt").getSystemObject());
    setTemplate(data.getUnscaledArray("Template").getLongArray());
    setUmsetzungsModul(data.getTextValue("UmsetzungsModul").getValueText());
    setAttributGruppe(data.getTextValue("AttributGruppe").getValueText());

    setVerwendeteAttributGruppenAspekte(new GlobalUmsetzungVerwendeteAttributGruppenAspekte(data.getTextArray("VerwendeteAttributGruppenAspekte")));
  }

  /**
   * Konstruktor mit Übergabe der einzelnen Attribut-Listen-Daten.
   * 
   * @param attributGruppe Information aus dem Attribut AttributGruppe der Attribut-Liste.
   * @param objekt Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   * @param template Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   * @param umsetzungsModul Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   */
  public GlobalUmsetzung(String attributGruppe, SystemObject objekt, long[] template, String umsetzungsModul)
  {
    m_attributGruppe = attributGruppe;
    m_objekt = objekt;
    m_template = template;
    m_umsetzungsModul = umsetzungsModul;
  }

  /**
   * Holen der Information aus dem Attribut AttributGruppe der Attribut-Liste.
   * 
   * @return Information aus dem Attribut AttributGruppe der Attribut-Liste.
   */
  public String getAttributGruppe()
  {
    return m_attributGruppe;
  }

  /**
   * Setzen der Information aus dem Attribut AttributGruppe der Attribut-Liste.
   * 
   * @param attributGruppe Information aus dem Attribut AttributGruppe der Attribut-Liste.
   */
  public void setAttributGruppe(String attributGruppe)
  {
    m_attributGruppe = attributGruppe;
  }

  /**
   * Holen der Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   * 
   * @return Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   */
  public SystemObject getObjekt()
  {
    return m_objekt;
  }

  /**
   * Setzen der Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   * 
   * @param objekt Information aus dem Attribut UmsetzungsObjekt der Attribut-Liste.
   */
  public void setObjekt(SystemObject objekt)
  {
    m_objekt = objekt;
  }

  /**
   * Holen der Informationen aus dem Attribut Template der Attribut-Liste.
   * 
   * @return Informationen aus dem Attribut Template der Attribut-Liste.
   */
  public long[] getTemplate()
  {
    return m_template;
  }

  /**
   * Setzen der Informationen aus dem Attribut Template der Attribut-Liste.
   * 
   * @param template Informationen aus dem Attribut Template der Attribut-Liste.
   */
  public void setTemplate(long[] template)
  {
    m_template = template;
  }

  /**
   * Holen der Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   * 
   * @return Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   */
  public String getUmsetzungsModul()
  {
    return m_umsetzungsModul;
  }

  /**
   * Setzen der Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   * 
   * @param umsetzungsModul Information aus dem Attribut UmsetzungsModul der Attribut-Liste.
   */
  public void setUmsetzungsModul(String umsetzungsModul)
  {
    if (umsetzungsModul.length() > 0)
    {
      m_umsetzungsModul = umsetzungsModul;
    }
  }

  /**
   * Holen der bei der Umsetzung zu verwendente AttributGruppen-Aspekt-Kombinationen.
   * 
   * @return bei der Umsetzung zu verwendente AttributGruppen-Aspekt-Kombinationen.
   */
  public GlobalUmsetzungVerwendeteAttributGruppenAspekte getVerwendeteAttributGruppenAspekte()
  {
    return m_verwendeteAttributGruppenAspekte;
  }

  /**
   * Setzen der bei der Umsetzung zu verwendente AttributGruppen-Aspekt-Kombinationen.
   * 
   * @param verwendeteAttributGruppenAspekte bei der Umsetzung zu verwendente AttributGruppen-Aspekt-Kombinationen.
   */
  public void setVerwendeteAttributGruppenAspekte(GlobalUmsetzungVerwendeteAttributGruppenAspekte verwendeteAttributGruppenAspekte)
  {
    m_verwendeteAttributGruppenAspekte = verwendeteAttributGruppenAspekte;
  }
}